﻿using System;
using System.Linq;
using System.Net;
using System.Net.Http;
using System.Web.Http;
using DotNetNuke.Web.Api;
using DotNetNuke.Security;
using DotNetNuclear.Modules.ClientCentric.Components;
using DotNetNuclear.Modules.ClientCentric.Components.Common;
using DotNetNuclear.Modules.ClientCentric.Entities;

namespace DotNetNuclear.Modules.ClientCentric.Services.Controllers
{
    [SupportedModules(Constants.DESKTOPMODULE_NAME)]
    public class ItemController : DnnApiController
    {
        IItemRepository _itemRepository;

        public ItemController()
        {
            _itemRepository = new ItemRepository();
        }

        /// <summary>
        /// API that returns Hello world
        /// </summary>
        /// <returns></returns>
        [HttpPost, HttpGet]  //[baseURL]/item/test
        [ActionName("test")]
        [AllowAnonymous]
        public HttpResponseMessage HelloWorld()
        {
            return Request.CreateResponse(HttpStatusCode.OK, "Hello World!");
        }

        /// <summary>
        /// API that creates a new item in the item list
        /// </summary>
        /// <returns></returns>
        [HttpPost]  //[baseURL]/item/new
        [ValidateAntiForgeryToken]
        [ActionName("new")]
        [DnnModuleAuthorize(AccessLevel = SecurityAccessLevel.Edit)]
        public HttpResponseMessage AddItem(RequestItem item)
        {
            try
            {
                Item newItem = new Item();
                newItem.ModuleId = ActiveModule.ModuleID;
                newItem.CreatedByUserId = UserInfo.UserID;
                newItem.CreatedOnDate = DateTime.Now;
                newItem.LastModifiedByUserId = UserInfo.UserID;
                newItem.LastModifiedOnDate = DateTime.Now;
                newItem.AssignedUserId = item.AssignedUserId;
                newItem.ItemName = item.ItemName;
                newItem.ItemDescription = item.ItemDescription;
                int itemId = _itemRepository.CreateItem(newItem);
                return Request.CreateResponse(HttpStatusCode.OK, itemId);
            }
            catch (Exception ex)
            {
                return Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message);
            }
        }

        /// <summary>
        /// API that deletes an item from the item list
        /// </summary>
        /// <returns></returns>
        [HttpPost]  //[baseURL]/item/delete
        [ValidateAntiForgeryToken]
        [ActionName("delete")]
        [DnnModuleAuthorize(AccessLevel = SecurityAccessLevel.Edit)]
        public HttpResponseMessage DeleteItem(RequestItem delItem)
        {
            try
            {
                _itemRepository.DeleteItem(delItem.ItemId, ActiveModule.ModuleID);
                return Request.CreateResponse(HttpStatusCode.OK, true.ToString());
            }
            catch (Exception ex)
            {
                return Request.CreateResponse(HttpStatusCode.NotFound, ex.Message);
            }
        }

        /// <summary>
        /// API that creates a new item in the item list
        /// </summary>
        /// <returns></returns>
        [HttpPost]  //[baseURL]/item/new
        [ValidateAntiForgeryToken]
        [ActionName("edit")]
        [DnnModuleAuthorize(AccessLevel = SecurityAccessLevel.Edit)]
        public HttpResponseMessage UpdateItem(Item item)
        {
            try
            {
                item.LastModifiedByUserId = UserInfo.UserID;
                item.LastModifiedOnDate = DateTime.Now;
                _itemRepository.UpdateItem(item);
                return Request.CreateResponse(HttpStatusCode.OK, true.ToString());
            }
            catch (Exception ex)
            {
                return Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message);
            }
        }

        /// <summary>
        /// API that returns an item list
        /// </summary>
        /// <returns></returns>
        [HttpPost,HttpGet]  //[baseURL]/item/list
        [ValidateAntiForgeryToken]
        [ActionName("list")]
        [DnnModuleAuthorize(AccessLevel = SecurityAccessLevel.View)]
        public HttpResponseMessage GetModuleItems()
        {
            try
            {
                var itemList = _itemRepository.GetItems(ActiveModule.ModuleID);
                return Request.CreateResponse(HttpStatusCode.OK, itemList.ToList());
            }
            catch (Exception ex)
            {
                return Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message);
            }
        }

        /// <summary>
        /// API that returns a single item
        /// </summary>
        /// <returns></returns>
        [HttpPost, HttpGet]  //[baseURL]/item/byid
        [ValidateAntiForgeryToken]
        [ActionName("byid")]
        [DnnModuleAuthorize(AccessLevel = SecurityAccessLevel.View)]
        public HttpResponseMessage GetItem(RequestById itemReq)
        {
            try
            {
                var item = _itemRepository.GetItem(itemReq.ItemId, ActiveModule.ModuleID);
                return Request.CreateResponse(HttpStatusCode.OK, item);
            }
            catch (Exception ex)
            {
                return Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message);
            }
        }

    }

    public class RequestItem
    {
        public int ItemId { get; set; }
        public string ItemName { get; set; }
        public string ItemDescription { get; set; }
        public int AssignedUserId { get; set; }
        public int CreatedByUserId { get; set; }
        public DateTime CreatedCreatedOnDate { get; set; }
    }

    public class RequestById
    {
        public int ItemId { get; set; }
    }

}